@@ -81,6 +81,9 @@ static int help(void)
 	printf("  contents        show contents of all controls for given card\n");
 	printf("  cset cID P      set control contents for one control\n");
 	printf("  cget cID        get control contents for one control\n");
+	printf("\nAvailable advanced commands:\n");
+	printf("  sevents	  show the mixer events for simple controls\n");
+	printf("  events	  show the mixer events for simple controls\n");
 	return 0;
 }
 
@@ -1602,7 +1605,8 @@ static int events(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 		if (res >= 0) {
 			printf("Poll ok: %i\n", res);
 			res = snd_hctl_handle_events(handle);
-			assert(res > 0);
+			if (res < 0)
+				printf("ERR: %s (%d)\n", snd_strerror(res), res);
 		}
 	}
 	snd_hctl_close(handle);
@@ -1771,7 +1775,7 @@ static int exec_stdin(void)
 
 int main(int argc, char *argv[])
 {
-	int morehelp, level = 0;
+	int badopt, retval, level = 0;
 	int read_stdin = 0;
 	static const struct option long_option[] =
 	{
@@ -1790,7 +1794,7 @@ int main(int argc, char *argv[])
 		{NULL, 0, NULL, 0},
 	};
 
-	morehelp = 0;
+	badopt = 0;
 	while (1) {
 		int c;
 
@@ -1805,10 +1809,14 @@ int main(int argc, char *argv[])
 				int i;
 				i = snd_card_get_index(optarg);
 				if (i >= 0 && i < 32)
+#if defined(SND_LIB_VER) && SND_LIB_VER(1, 2, 5) <= SND_LIB_VERSION
+					sprintf(card, "sysdefault:%i", i);
+#else
 					sprintf(card, "hw:%i", i);
+#endif
 				else {
-					fprintf(stderr, "Invalid card number.\n");
-					morehelp++;
+					fprintf(stderr, "Invalid card number '%s'.\n", optarg);
+					badopt++;
 				}
 			}
 			break;
@@ -1830,7 +1838,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'v':
 			printf("amixer version " SND_UTIL_VERSION_STR "\n");
-			return 1;
+			return 0;
 		case 'a':
 			smixer_level = 1;
 			memset(&smixer_options, 0, sizeof(smixer_options));
@@ -1841,7 +1849,7 @@ int main(int argc, char *argv[])
 				smixer_options.abstract = SND_MIXER_SABSTRACT_BASIC;
 			else {
 				fprintf(stderr, "Select correct abstraction level (none or basic)...\n");
-				morehelp++;
+				badopt++;
 			}
 			break;
 		case 's':
@@ -1854,49 +1862,55 @@ int main(int argc, char *argv[])
 			std_vol_type = VOL_MAP;
 			break;
 		default:
-			fprintf(stderr, "Invalid switch or option needs an argument.\n");
-			morehelp++;
+			fprintf(stderr, "Invalid switch or option -%c needs an argument.\n", c);
+			badopt++;
 		}
 	}
-	if (morehelp) {
-		help();
+	if (badopt)
 		return 1;
-	}
+
 	smixer_options.device = card;
 
-	if (read_stdin)
-		return exec_stdin();
+	if (read_stdin) {
+		retval = exec_stdin();
+		goto finish;
+	}
 
 	if (argc - optind <= 0) {
-		return selems(LEVEL_BASIC | level) ? 1 : 0;
+		retval = selems(LEVEL_BASIC | level) ? 1 : 0;
+		goto finish;
 	}
 	if (!strcmp(argv[optind], "help")) {
-		return help() ? 1 : 0;
+		retval = help() ? 1 : 0;
 	} else if (!strcmp(argv[optind], "info")) {
-		return info() ? 1 : 0;
+		retval = info() ? 1 : 0;
 	} else if (!strcmp(argv[optind], "controls")) {
-		return controls(level) ? 1 : 0;
+		retval = controls(level) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "contents")) {
-		return controls(LEVEL_BASIC | level) ? 1 : 0;
+		retval = controls(LEVEL_BASIC | level) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "scontrols") || !strcmp(argv[optind], "simple")) {
-		return selems(level) ? 1 : 0;
+		retval = selems(level) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "scontents")) {
-		return selems(LEVEL_BASIC | level) ? 1 : 0;
+		retval = selems(LEVEL_BASIC | level) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "sset") || !strcmp(argv[optind], "set")) {
-		return sset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 0, 0) ? 1 : 0;
+		retval = sset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 0, 0) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "sget") || !strcmp(argv[optind], "get")) {
-		return sset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 1, 0) ? 1 : 0;
+		retval = sset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 1, 0) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "cset")) {
-		return cset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 0, 0) ? 1 : 0;
+		retval = cset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 0, 0) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "cget")) {
-		return cset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 1, 0) ? 1 : 0;
+		retval = cset(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL, 1, 0) ? 1 : 0;
 	} else if (!strcmp(argv[optind], "events")) {
-		return events(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL);
+		retval = events(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL);
 	} else if (!strcmp(argv[optind], "sevents")) {
-		return sevents(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL);
+		retval = sevents(argc - optind - 1, argc - optind > 1 ? argv + optind + 1 : NULL);
 	} else {
 		fprintf(stderr, "amixer: Unknown command '%s'...\n", argv[optind]);
+		retval = 0;
 	}
 
-	return 0;
+finish:
+	snd_config_update_free_global();
+
+	return retval;
 }
