@@ -24,6 +24,7 @@
 #include <math.h>
 #include <limits.h>
 #include <locale.h>
+#include <math.h>
 
 #include "aconfig.h"
 #include "gettext.h"
@@ -31,19 +32,60 @@
 
 #include "common.h"
 
+#ifdef HAVE_LIBTINYALSA
+#include "tinyalsa.h"
+#else
 #include "alsa.h"
+#endif
 #include "convert.h"
+#ifdef HAVE_LIBFFTW3F
 #include "analyze.h"
+#endif
+#include "latencytest.h"
+
+/* get snr threshold in dB */
+static void get_snr_thd_db(struct bat *bat, char *thd)
+{
+	int err;
+	float thd_db;
+	char *ptrf;
+
+	thd_db = strtof(thd, &ptrf);
+	err = -errno;
+	if (!snr_is_valid(thd_db)) {
+		fprintf(bat->err, _("Invalid threshold '%s':%d\n"), thd, err);
+		exit(EXIT_FAILURE);
+	}
+	bat->snr_thd_db = thd_db;
+}
+
+/* get snr threshold in %, and convert to dB */
+static void get_snr_thd_pc(struct bat *bat, char *thd)
+{
+	int err;
+	float thd_pc;
+	char *ptrf;
+
+	thd_pc = strtof(thd, &ptrf);
+	err = -errno;
+	if (thd_pc <= 0.0 || thd_pc >= 100.0) {
+		fprintf(bat->err, _("Invalid threshold '%s':%d\n"), thd, err);
+		exit(EXIT_FAILURE);
+	}
+	bat->snr_thd_db = 20.0 * log10f(100.0 / thd_pc);
+}
 
 static int get_duration(struct bat *bat)
 {
+	int err;
 	float duration_f;
 	long duration_i;
 	char *ptrf, *ptri;
 
 	duration_f = strtof(bat->narg, &ptrf);
+	err = -errno;
 	if (duration_f == HUGE_VALF || duration_f == -HUGE_VALF
-			|| (duration_f == 0.0 && errno != 0))
+			|| (duration_f == 0.0 && err != 0))
 		goto err_exit;
 
 	duration_i = strtol(bat->narg, &ptri, 10);
@@ -59,16 +101,16 @@ static int get_duration(struct bat *bat)
 
 	if (bat->frames <= 0 || bat->frames > MAX_FRAMES) {
 		fprintf(bat->err, _("Invalid duration. Range: (0, %d(%fs))\n"),
-				MAX_FRAMES, (double)MAX_FRAMES / bat->rate);
+				MAX_FRAMES, (float)MAX_FRAMES / bat->rate);
 		return -EINVAL;
 	}
 
 	return 0;
 
 err_exit:
-	fprintf(bat->err, _("Duration overflow/underflow: %d\n"), -errno);
+	fprintf(bat->err, _("Duration overflow/underflow: %d\n"), err);
 
-	return -errno;
+	return err;
 }
 
 static void get_sine_frequencies(struct bat *bat, char *freq)
@@ -88,37 +130,30 @@ static void get_sine_frequencies(struct bat *bat, char *freq)
 static void get_format(struct bat *bat, char *optarg)
 {
 	if (strcasecmp(optarg, "cd") == 0) {
-		bat->format = SND_PCM_FORMAT_S16_LE;
+		bat->format = BAT_PCM_FORMAT_S16_LE;
 		bat->rate = 44100;
 		bat->channels = 2;
+		bat->sample_size = 2;
 	} else if (strcasecmp(optarg, "dat") == 0) {
-		bat->format = SND_PCM_FORMAT_S16_LE;
+		bat->format = BAT_PCM_FORMAT_S16_LE;
 		bat->rate = 48000;
 		bat->channels = 2;
-	} else {
-		bat->format = snd_pcm_format_value(optarg);
-		if (bat->format == SND_PCM_FORMAT_UNKNOWN) {
-			fprintf(bat->err, _("wrong extended format '%s'\n"),
-					optarg);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	switch (bat->format) {
-	case SND_PCM_FORMAT_U8:
-		bat->sample_size = 1;
-		break;
-	case SND_PCM_FORMAT_S16_LE:
 		bat->sample_size = 2;
-		break;
-	case SND_PCM_FORMAT_S24_3LE:
+	} else if (strcasecmp(optarg, "U8") == 0) {
+		bat->format = BAT_PCM_FORMAT_U8;
+		bat->sample_size = 1;
+	} else if (strcasecmp(optarg, "S16_LE") == 0) {
+		bat->format = BAT_PCM_FORMAT_S16_LE;
+		bat->sample_size = 2;
+	} else if (strcasecmp(optarg, "S24_3LE") == 0) {
+		bat->format = BAT_PCM_FORMAT_S24_3LE;
 		bat->sample_size = 3;
-		break;
-	case SND_PCM_FORMAT_S32_LE:
+	} else if (strcasecmp(optarg, "S32_LE") == 0) {
+		bat->format = BAT_PCM_FORMAT_S32_LE;
 		bat->sample_size = 4;
-		break;
-	default:
-		fprintf(bat->err, _("unsupported format: %d\n"), bat->format);
+	} else {
+		bat->format = BAT_PCM_FORMAT_UNKNOWN;
+		fprintf(bat->err, _("wrong extended format '%s'\n"), optarg);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -191,6 +226,12 @@ static void test_loopback(struct bat *bat)
 		exit(EXIT_FAILURE);
 	}
 
+	/* check if capture thread is canceled or not */
+	if (thread_result_capture == PTHREAD_CANCELED) {
+		fprintf(bat->log, _("Capture canceled.\n"));
+		return;
+	}
+
 	/* check capture status */
 	if (*thread_result_capture != 0) {
 		fprintf(bat->err, _("Exit capture thread fail: %d\n"),
@@ -272,7 +313,7 @@ static void test_capture(struct bat *bat)
 static void usage(struct bat *bat)
 {
 	fprintf(bat->log,
-_("Usage: bat [-options]...\n"
+_("Usage: alsabat [-options]...\n"
 "\n"
 "  -h, --help             this help\n"
 "  -D                     pcm device for both playback and capture\n"
@@ -285,16 +326,19 @@ _("Usage: bat [-options]...\n"
 "  -k                     parameter for frequency detecting threshold\n"
 "  -F                     target frequency\n"
 "  -p                     total number of periods to play/capture\n"
+"  -B                     buffer size in frames\n"
+"  -E                     period size in frames\n"
 "      --log=#            file that both stdout and strerr redirecting to\n"
 "      --file=#           file for playback\n"
 "      --saveplay=#       file that storing playback content, for debug\n"
 "      --local            internal loop, set to bypass pcm hardware devices\n"
+"      --standalone       standalone mode, to bypass analysis\n"
+"      --roundtriplatency round trip latency mode\n"
+"      --snr-db=#         noise detect threshold, in SNR(dB)\n"
+"      --snr-pc=#         noise detect threshold, in noise percentage(%%)\n"
 ));
-	fprintf(bat->log, _("Recognized sample formats are: %s %s %s %s\n"),
-			snd_pcm_format_name(SND_PCM_FORMAT_U8),
-			snd_pcm_format_name(SND_PCM_FORMAT_S16_LE),
-			snd_pcm_format_name(SND_PCM_FORMAT_S24_3LE),
-			snd_pcm_format_name(SND_PCM_FORMAT_S32_LE));
+	fprintf(bat->log, _("Recognized sample formats are: "));
+	fprintf(bat->log, _("U8 S16_LE S24_3LE S32_LE\n"));
 	fprintf(bat->log, _("The available format shotcuts are:\n"));
 	fprintf(bat->log, _("-f cd (16 bit little endian, 44100, stereo)\n"));
 	fprintf(bat->log, _("-f dat (16 bit little endian, 48000, stereo)\n"));
@@ -306,22 +350,32 @@ static void set_defaults(struct bat *bat)
 
 	/* Set default values */
 	bat->rate = 44100;
-	bat->channels = 1;
 	bat->frame_size = 2;
 	bat->sample_size = 2;
-	bat->format = SND_PCM_FORMAT_S16_LE;
+	bat->format = BAT_PCM_FORMAT_S16_LE;
 	bat->convert_float_to_sample = convert_float_to_int16;
-	bat->convert_sample_to_double = convert_int16_to_double;
+	bat->convert_sample_to_float = convert_int16_to_float;
 	bat->frames = bat->rate * 2;
 	bat->target_freq[0] = 997.0;
 	bat->target_freq[1] = 997.0;
 	bat->sigma_k = 3.0;
+	bat->snr_thd_db = SNR_DB_INVALID;
 	bat->playback.device = NULL;
 	bat->capture.device = NULL;
 	bat->buf = NULL;
 	bat->local = false;
+	bat->buffer_size = 0;
+	bat->period_size = 0;
+	bat->roundtriplatency = false;
+#ifdef HAVE_LIBTINYALSA
+	bat->channels = 2;
+	bat->playback.fct = &playback_tinyalsa;
+	bat->capture.fct = &record_tinyalsa;
+#else
+	bat->channels = 1;
 	bat->playback.fct = &playback_alsa;
 	bat->capture.fct = &record_alsa;
+#endif
 	bat->playback.mode = MODE_LOOPBACK;
 	bat->capture.mode = MODE_LOOPBACK;
 	bat->period_is_limited = false;
@@ -331,14 +385,18 @@ static void set_defaults(struct bat *bat)
 
 static void parse_arguments(struct bat *bat, int argc, char *argv[])
 {
-	int c, option_index;
-	static const char short_options[] = "D:P:C:f:n:F:c:r:s:k:p:lth";
+	int c, option_index, err;
+	static const char short_options[] = "D:P:C:f:n:F:c:r:s:k:p:B:E:lth";
 	static const struct option long_options[] = {
 		{"help",     0, 0, 'h'},
 		{"log",      1, 0, OPT_LOG},
 		{"file",     1, 0, OPT_READFILE},
 		{"saveplay", 1, 0, OPT_SAVEPLAY},
 		{"local",    0, 0, OPT_LOCAL},
+		{"standalone", 0, 0, OPT_STANDALONE},
+		{"roundtriplatency", 0, 0, OPT_ROUNDTRIPLATENCY},
+		{"snr-db",   1, 0, OPT_SNRTHD_DB},
+		{"snr-pc",   1, 0, OPT_SNRTHD_PC},
 		{0, 0, 0, 0}
 	};
 
@@ -357,6 +415,18 @@ static void parse_arguments(struct bat *bat, int argc, char *argv[])
 		case OPT_LOCAL:
 			bat->local = true;
 			break;
+		case OPT_STANDALONE:
+			bat->standalone = true;
+			break;
+		case OPT_ROUNDTRIPLATENCY:
+			bat->roundtriplatency = true;
+			break;
+		case OPT_SNRTHD_DB:
+			get_snr_thd_db(bat, optarg);
+			break;
+		case OPT_SNRTHD_PC:
+			get_snr_thd_pc(bat, optarg);
+			break;
 		case 'D':
 			if (bat->playback.device == NULL)
 				bat->playback.device = optarg;
@@ -399,6 +469,16 @@ static void parse_arguments(struct bat *bat, int argc, char *argv[])
 			bat->periods_total = atoi(optarg);
 			bat->period_is_limited = true;
 			break;
+		case 'B':
+			err = atoi(optarg);
+			bat->buffer_size = err >= MIN_BUFFERSIZE
+					&& err < MAX_BUFFERSIZE ? err : 0;
+			break;
+		case 'E':
+			err = atoi(optarg);
+			bat->period_size = err >= MIN_PERIODSIZE
+					&& err < MAX_PERIODSIZE ? err : 0;
+			break;
 		case 'h':
 		default:
 			usage(bat);
@@ -451,17 +531,18 @@ static int validate_options(struct bat *bat)
 static int bat_init(struct bat *bat)
 {
 	int err = 0;
+	int fd = 0;
 	char name[] = TEMP_RECORD_FILE_NAME;
 
 	/* Determine logging to a file or stdout and stderr */
 	if (bat->logarg) {
 		bat->log = NULL;
 		bat->log = fopen(bat->logarg, "wb");
+		err = -errno;
 		if (bat->log == NULL) {
-			fprintf(bat->err, _("Cannot open file for capture:"));
-			fprintf(bat->err, _(" %s %d\n"),
-					bat->logarg, -errno);
-			return -errno;
+			fprintf(bat->err, _("Cannot open file: %s %d\n"),
+					bat->logarg, err);
+			return err;
 		}
 		bat->err = bat->log;
 	}
@@ -473,23 +554,29 @@ static int bat_init(struct bat *bat)
 			return err;
 	}
 
+	/* Set default playback and capture devices */
+	if (bat->playback.device == NULL && bat->capture.device == NULL)
+		bat->playback.device = bat->capture.device = DEFAULT_DEV_NAME;
+
 	/* Determine capture file */
 	if (bat->local) {
 		bat->capture.file = bat->playback.file;
 	} else {
 		/* create temp file for sound record and analysis */
-		err = mkstemp(name);
-		if (err == -1) {
+		fd = mkstemp(name);
+		err = -errno;
+		if (fd == -1) {
 			fprintf(bat->err, _("Fail to create record file: %d\n"),
-					-errno);
-			return -errno;
+					err);
+			return err;
 		}
 		/* store file name which is dynamically created */
 		bat->capture.file = strdup(name);
+		err = -errno;
 		if (bat->capture.file == NULL)
-			return -errno;
+			return err;
 		/* close temp file */
-		close(err);
+		close(fd);
 	}
 
 	/* Initial for playback */
@@ -515,11 +602,11 @@ static int bat_init(struct bat *bat)
 		}
 	} else {
 		bat->fp = fopen(bat->playback.file, "rb");
+		err = -errno;
 		if (bat->fp == NULL) {
-			fprintf(bat->err, _("Cannot open file for playback:"));
-			fprintf(bat->err, _(" %s %d\n"),
-					bat->playback.file, -errno);
-			return -errno;
+			fprintf(bat->err, _("Cannot open file: %s %d\n"),
+					bat->playback.file, err);
+			return err;
 		}
 		err = read_wav_header(bat, bat->playback.file, bat->fp, false);
 		fclose(bat->fp);
@@ -533,19 +620,19 @@ static int bat_init(struct bat *bat)
 	switch (bat->sample_size) {
 	case 1:
 		bat->convert_float_to_sample = convert_float_to_uint8;
-		bat->convert_sample_to_double = convert_uint8_to_double;
+		bat->convert_sample_to_float = convert_uint8_to_float;
 		break;
 	case 2:
 		bat->convert_float_to_sample = convert_float_to_int16;
-		bat->convert_sample_to_double = convert_int16_to_double;
+		bat->convert_sample_to_float = convert_int16_to_float;
 		break;
 	case 3:
 		bat->convert_float_to_sample = convert_float_to_int24;
-		bat->convert_sample_to_double = convert_int24_to_double;
+		bat->convert_sample_to_float = convert_int24_to_float;
 		break;
 	case 4:
 		bat->convert_float_to_sample = convert_float_to_int32;
-		bat->convert_sample_to_double = convert_int32_to_double;
+		bat->convert_sample_to_float = convert_int32_to_float;
 		break;
 	default:
 		fprintf(bat->err, _("Invalid PCM format: size=%d\n"),
@@ -580,6 +667,35 @@ int main(int argc, char *argv[])
 	if (err < 0)
 		goto out;
 
+	/* round trip latency test thread */
+	if (bat.roundtriplatency) {
+		while (1) {
+			fprintf(bat.log,
+				_("\nStart round trip latency\n"));
+			roundtrip_latency_init(&bat);
+			test_loopback(&bat);
+
+			if (bat.latency.xrun_error == false)
+				break;
+			else {
+				/* Xrun error in playback or capture,
+				increase period size and try again */
+				bat.period_size += bat.rate / 1000;
+				bat.buffer_size =
+					bat.period_size * DIV_BUFFERSIZE;
+
+				/* terminate the test if period_size is
+				large enough */
+				if (bat.period_size > bat.rate * 0.2)
+					break;
+			}
+
+			/* Waiting 500ms and start the next round */
+			usleep(CAPTURE_DELAY * 1000);
+		}
+		goto out;
+	}
+
 	/* single line playback thread: playback only, no capture */
 	if (bat.playback.mode == MODE_SINGLE) {
 		test_playback(&bat);
@@ -597,7 +713,12 @@ int main(int argc, char *argv[])
 		test_loopback(&bat);
 
 analyze:
-	err = analyze_capture(&bat);
+#ifdef HAVE_LIBFFTW3F
+	if (!bat.standalone || snr_is_valid(bat.snr_thd_db))
+		err = analyze_capture(&bat);
+#else
+	fprintf(bat.log, _("No libfftw3 library. Exit without analysis.\n"));
+#endif
 out:
 	fprintf(bat.log, _("\nReturn value is %d\n"), err);
 
