@@ -14,7 +14,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <stdio.h>
@@ -52,13 +52,13 @@ struct cmdtbl {
 	const char *desc;
 };
 
-static struct cmdtbl cmds[] = {
+static const struct cmdtbl cmds[] = {
 	{ "pro", IDX_PRO, CMD_BOOL,
 	  "professional (common)\n\toff = consumer mode, on = professional mode" },
 	{ "aud", IDX_NOAUDIO, CMD_BOOL_INV,
 	  "audio (common)\n\ton = audio mode, off = non-audio mode" },
 	{ "rat", IDX_RATE, CMD_INT,
-	  "rate (common)\n\tsample rate in Hz" },
+	  "rate (common)\n\tsample rate in Hz (0 = not indicated)" },
 	{ "emp", IDX_EMP, CMD_INT,
 	  "emphasis (common)\n\t0 = none, 1 = 50/15us, 2 = CCITT" },
 	{ "loc", IDX_UNLOCK, CMD_BOOL_INV,
@@ -90,6 +90,7 @@ static void usage(void)
 	printf("Options:\n");
 	printf("    -D device   specifies the control device to use\n");
 	printf("    -c card     specifies the card number to use (equiv. with -Dhw:#)\n");
+	printf("    -n number   specifies the control index number (default = 0)\n");
 	printf("    -x          dump the dump the AESx hex code for IEC958 PCM parameters\n");
 	printf("    -i          read commands from stdin\n");
 	printf("Commands:\n");
@@ -194,14 +195,38 @@ static int update_iec958_status(snd_aes_iec958_t *iec958, int *parms)
 		} else {
 			iec958->status[3] &= ~IEC958_AES3_CON_FS;
 			switch (parms[IDX_RATE]) {
+			case 22050:
+				iec958->status[3] |= IEC958_AES3_CON_FS_22050;
+				break;
+			case 24000:
+				iec958->status[3] |= IEC958_AES3_CON_FS_24000;
+				break;
+			case 32000:
+				iec958->status[3] |= IEC958_AES3_CON_FS_32000;
+				break;
 			case 44100:
 				iec958->status[3] |= IEC958_AES3_CON_FS_44100;
 				break;
 			case 48000:
 				iec958->status[3] |= IEC958_AES3_CON_FS_48000;
 				break;
-			case 32000:
-				iec958->status[3] |= IEC958_AES3_CON_FS_32000;
+			case 88200:
+				iec958->status[3] |= IEC958_AES3_CON_FS_88200;;
+				break;
+			case 96000:
+				iec958->status[3] |= IEC958_AES3_CON_FS_96000;
+				break;
+			case 176400:
+				iec958->status[3] |= IEC958_AES3_CON_FS_176400;
+				break;
+			case 192000:
+				iec958->status[3] |= IEC958_AES3_CON_FS_192000;
+				break;
+			case 768000:
+				iec958->status[3] |= IEC958_AES3_CON_FS_768000;
+				break;
+			default:
+				iec958->status[3] |= IEC958_AES3_CON_FS_NOTID;
 				break;
 			}
 		}
@@ -286,6 +311,7 @@ int main(int argc, char **argv)
 {
 	const char *dev = "default";
 	const char *spdif_str = SND_CTL_NAME_IEC958("", PLAYBACK, DEFAULT);
+	int spdif_index = -1;
 	snd_ctl_t *ctl;
 	snd_ctl_elem_list_t *clist;
 	snd_ctl_elem_id_t *cid;
@@ -301,20 +327,23 @@ int main(int argc, char **argv)
 	for (i = 0; i < IDX_LAST; i++)
 		parms[i] = -1; /* not set */
 
-	while ((c = getopt(argc, argv, "D:c:xhi")) != -1) {
+	while ((c = getopt(argc, argv, "D:c:n:xhi")) != -1) {
 		switch (c) {
 		case 'D':
 			dev = optarg;
 			break;
 		case 'c':
 			i = atoi(optarg);
-			if (i < 0 || i >= 7) {
+			if (i < 0 || i >= 32) {
 				fprintf(stderr, "invalid card index %d\n", i);
 				return 1;
 			}
 			sprintf(tmpname, "hw:%d", i);
 			dev = tmpname;
 			break;
+		case 'n':
+			spdif_index = atoi(optarg);
+			break;
 		case 'x':
 			dumphex = 1;
 			break;
@@ -349,10 +378,13 @@ int main(int argc, char **argv)
 	controls = snd_ctl_elem_list_get_used(clist);
 	for (cidx = 0; cidx < controls; cidx++) {
 		if (!strcmp(snd_ctl_elem_list_get_name(clist, cidx), spdif_str))
-			break;
+			if (spdif_index < 0 ||
+			    snd_ctl_elem_list_get_index(clist, cidx) == spdif_index)
+				break;
 	}
 	if (cidx >= controls) {
-		fprintf(stderr, "control \"%s\" not found\n", spdif_str);
+		fprintf(stderr, "control \"%s\" (index %d) not found\n",
+			spdif_str, spdif_index);
 		return 1;
 	}
 
