@@ -20,6 +20,7 @@
   in the file called LICENSE.GPL.
 */
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -35,6 +36,9 @@
 #include <alsa/topology.h>
 #include "gettext.h"
 #include "version.h"
+#include "topology.h"
+
+bool pre_process_config = false;
 
 static snd_output_t *log;
 
@@ -45,11 +49,16 @@ _("Usage: %s [OPTIONS]...\n"
 "\n"
 "-h, --help              help\n"
 "-c, --compile=FILE      compile configuration file\n"
+"-p, --pre-process       pre-process Topology2.0 configuration file before compilation\n"
+"-P, --pre-process=FILE  pre-process Topology2.0 configuration file\n"
 "-d, --decode=FILE       decode binary topology file\n"
 "-n, --normalize=FILE    normalize configuration file\n"
 "-u, --dump=FILE         dump (reparse) configuration file\n"
 "-v, --verbose=LEVEL     set verbosity level (0...1)\n"
 "-o, --output=FILE       set output file\n"
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+"-D, --define=ARGS       define variables (VAR1=VAL1[,VAR2=VAL2] ...)\n"
+#endif
 "-s, --sort              sort the identifiers in the normalized output\n"
 "-g, --group             save configuration by group indexes\n"
 "-x, --nocheck           save configuration without additional integrity checks\n"
@@ -100,10 +109,8 @@ static int load(const char *source_file, void **dst, size_t *dst_size)
 		pos += r;
 		size += 8*1024;
 		buf2 = realloc(buf, size);
-		if (buf2 == NULL) {
-			free(buf);
+		if (buf2 == NULL)
 			goto _nomem;
-		}
 		buf = buf2;
 	}
 	if (r < 0) {
@@ -229,10 +236,53 @@ static int dump(const char *source_file, const char *output_file, int cflags, in
 	return err;
 }
 
-static int compile(const char *source_file, const char *output_file, int cflags)
+static char *get_inc_path(const char *filename)
 {
+	const char *s = strrchr(filename, '/');
+	char *r = strdup(filename);
+	if (r && s)
+		r[s - filename] = '\0';
+	return r;
+}
+
+/* Convert Topology2.0 conf to the existing conf syntax */
+static int pre_process_conf(const char *source_file, const char *output_file,
+			    const char *pre_processor_defs)
+{
+	struct tplg_pre_processor *tplg_pp;
+	size_t config_size;
+	char *config, *inc_path;
+	int err;
+
+	err = load(source_file, (void **)&config, &config_size);
+	if (err)
+		return err;
+
+	/* init pre-processor */
+	err = init_pre_processor(&tplg_pp, SND_OUTPUT_STDIO, output_file);
+	if (err < 0) {
+		fprintf(stderr, _("failed to init pre-processor for Topology2.0\n"));
+		free(config);
+		return err;
+	}
+
+	/* pre-process conf file */
+	inc_path = get_inc_path(source_file);
+	err = pre_process(tplg_pp, config, config_size, pre_processor_defs, inc_path);
+	free(inc_path);
+
+	/* free pre-processor */
+	free_pre_preprocessor(tplg_pp);
+	free(config);
+	return err;
+}
+
+static int compile(const char *source_file, const char *output_file, int cflags,
+		   const char *pre_processor_defs)
+{
+	struct tplg_pre_processor *tplg_pp = NULL;
 	snd_tplg_t *tplg;
-	char *config;
+	char *config, *inc_path;
 	void *bin;
 	size_t config_size, size;
 	int err;
@@ -240,7 +290,34 @@ static int compile(const char *source_file, const char *output_file, int cflags)
 	err = load(source_file, (void **)&config, &config_size);
 	if (err)
 		return err;
-	err = load_topology(&tplg, config, config_size, cflags);
+
+	/* pre-process before compiling */
+	if (pre_process_config) {
+		char *pconfig;
+		size_t size;
+
+		/* init pre-processor */
+		init_pre_processor(&tplg_pp, SND_OUTPUT_BUFFER, NULL);
+
+		/* pre-process conf file */
+		inc_path = get_inc_path(source_file);
+		err = pre_process(tplg_pp, config, config_size, pre_processor_defs, inc_path);
+		free(inc_path);
+		if (err) {
+			free_pre_preprocessor(tplg_pp);
+			free(config);
+			return err;
+		}
+
+		/* load topology */
+		size = snd_output_buffer_string(tplg_pp->output, &pconfig);
+		err = load_topology(&tplg, pconfig, size, cflags);
+
+		/* free pre-processor */
+		free_pre_preprocessor(tplg_pp);
+	} else {
+		err = load_topology(&tplg, config, config_size, cflags);
+	}
 	free(config);
 	if (err)
 		return err;
@@ -294,15 +371,23 @@ static int decode(const char *source_file, const char *output_file,
 
 int main(int argc, char *argv[])
 {
-	static const char short_options[] = "hc:d:n:u:v:o:sgxzV";
+	static const char short_options[] = "hc:d:n:u:v:o:pP:sgxzV"
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+		"D:"
+#endif
+		;
 	static const struct option long_options[] = {
 		{"help", 0, NULL, 'h'},
 		{"verbose", 1, NULL, 'v'},
 		{"compile", 1, NULL, 'c'},
+		{"pre-process", 1, NULL, 'p'},
 		{"decode", 1, NULL, 'd'},
 		{"normalize", 1, NULL, 'n'},
 		{"dump", 1, NULL, 'u'},
 		{"output", 1, NULL, 'o'},
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+		{"define", 1, NULL, 'D'},
+#endif
 		{"sort", 0, NULL, 's'},
 		{"group", 0, NULL, 'g'},
 		{"nocheck", 0, NULL, 'x'},
@@ -312,6 +397,7 @@ int main(int argc, char *argv[])
 	};
 	char *source_file = NULL;
 	char *output_file = NULL;
+	const char *pre_processor_defs = NULL;
 	int c, err, op = 'c', cflags = 0, dflags = 0, sflags = 0, option_index;
 
 #ifdef ENABLE_NLS
@@ -338,7 +424,7 @@ int main(int argc, char *argv[])
 		case 'n':
 		case 'u':
 			if (source_file) {
-				fprintf(stderr, _("Cannot combine operations (compile, normalize, dump)\n"));
+				fprintf(stderr, _("Cannot combine operations (compile, normalize, pre-process, dump)\n"));
 				return 1;
 			}
 			source_file = optarg;
@@ -350,12 +436,24 @@ int main(int argc, char *argv[])
 		case 's':
 			sflags |= SND_TPLG_SAVE_SORT;
 			break;
+		case 'P':
+			op = 'P';
+			source_file = optarg;
+			break;
+		case 'p':
+			pre_process_config = true;
+			break;
 		case 'g':
 			sflags |= SND_TPLG_SAVE_GROUPS;
 			break;
 		case 'x':
 			sflags |= SND_TPLG_SAVE_NOCHECK;
 			break;
+#if SND_LIB_VER(1, 2, 5) < SND_LIB_VERSION
+		case 'D':
+			pre_processor_defs = optarg;
+			break;
+#endif
 		case 'V':
 			version(argv[0]);
 			return 0;
@@ -381,11 +479,14 @@ int main(int argc, char *argv[])
 
 	switch (op) {
 	case 'c':
-		err = compile(source_file, output_file, cflags);
+		err = compile(source_file, output_file, cflags, pre_processor_defs);
 		break;
 	case 'd':
 		err = decode(source_file, output_file, cflags, dflags, sflags);
 		break;
+	case 'P':
+		err = pre_process_conf(source_file, output_file, pre_processor_defs);
+		break;
 	default:
 		err = dump(source_file, output_file, cflags, sflags);
 		break;
